/**
 * A Sole Topic Setter is a layer between producing a message that sets some data, but ignores the update if there is no change.
 * First it consumes all messages in the topic and stores them by logCompactId
 * Then when we want to set the same id to the same payload value, it ignores it to prevent unnecessary messages.
 * This class is very useful in a singleton processor that writes to a single target topic.
 * Due to the async nature of how the streams are processed, the local memoryHash object can become out of date if other processes are writing to the target topic
 * Consequently, a sole topic setter expects to be the only thing writing topic data
 * Otherwise, something else could interfere with the setting of data and confuse the setter with race conditions.
 * When we write the topic data, we don't wait around for the confirmation
 */
import { TopicConsumer } from "./topicConsumer";
import { TopicProducer } from "./topicProducer";
export default class SoleTopicSetter<T> {
    private consumer;
    private producer;
    private memoryHash;
    private isReady;
    private messagesToApply;
    private messagesToPush;
    private isProducingMessages;
    private lastTriggered;
    private triggerInterval;
    private stopStreaming;
    constructor(consumer: TopicConsumer<T>, producer: TopicProducer<T>);
    start(): Promise<void>;
    stop(): Promise<void>;
    /**
     * Run this after sending bulk messages using autoFlush = false
     */
    triggerWaitingMessages(): Promise<void>;
    private produceWaitingMessages;
    private processMessage;
    setLogCompactedPayload(logCompactId: string, payload: T, autoFlush?: boolean): void;
    setPayload(payload: T, autoFlush?: boolean): void;
}
